@@ -18,9 +18,9 @@ def feval(parameters):
     elif parameters[0] == 'local_score_marginal_multi':
         return local_score_marginal_multi(parameters[1], parameters[2], parameters[3], parameters[4])
     elif parameters[0] == 'local_score_BIC':
-        return local_score_bic(parameters[1], parameters[2], parameters[3], parameters[4])
+        return local_score_BIC(parameters[1], parameters[2], parameters[3], parameters[4])
     elif parameters[0] == 'local_score_BDeu':
-        return local_score_bic(parameters[1], parameters[2], parameters[3], parameters[4])
+        return local_score_BDeu(parameters[1], parameters[2], parameters[3], parameters[4])
     else:
         raise Exception('Undefined function')
 
