@@ -3,7 +3,9 @@ from numpy import sqrt
 from numpy.linalg import eigh, eigvalsh
 from scipy import stats
 from sklearn.gaussian_process import GaussianProcessRegressor
-from sklearn.gaussian_process.kernels import RBF, WhiteKernel, ConstantKernel as C
+from sklearn.gaussian_process.kernels import RBF
+from sklearn.gaussian_process.kernels import ConstantKernel as C
+from sklearn.gaussian_process.kernels import WhiteKernel
 
 from causallearn.utils.KCI.GaussianKernel import GaussianKernel
 from causallearn.utils.KCI.Kernel import Kernel
@@ -15,7 +17,7 @@ from causallearn.utils.KCI.PolynomialKernel import PolynomialKernel
 class KCI_UInd(object):
     '''
     Python implementation of Kernel-based Conditional Independence (KCI) test. Unconditional version.
-    The official Matlab implementation can be found in http://people.tuebingen.mpg.de/kzhang/KCI-test.zip
+    The original Matlab implementation can be found in http://people.tuebingen.mpg.de/kzhang/KCI-test.zip
 
     References
     ----------
@@ -179,7 +181,7 @@ class KCI_UInd(object):
 class KCI_CInd(object):
     '''
     Python implementation of Kernel-based Conditional Independence (KCI) test. Conditional version.
-    The official Matlab implementation can be found in http://people.tuebingen.mpg.de/kzhang/KCI-test.zip
+    The original Matlab implementation can be found in http://people.tuebingen.mpg.de/kzhang/KCI-test.zip
 
     References
     ----------
